@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2023 NRK and contributors.
+ * Copyright (C) 2022-2024 NRK and contributors.
  *
  * This file is part of sxcs.
  *
@@ -41,7 +41,7 @@
 	#define PROGNAME "sxcs"
 #endif
 #ifndef VERSION
-	#define VERSION "v0.7.3"
+	#define VERSION "v1.0.0"
 #endif
 
 #define ARRLEN(X)        (sizeof(X) / sizeof((X)[0]))
@@ -114,6 +114,7 @@ typedef struct {
 	uint oneshot           : 1;
 	uint quit_on_keypress  : 1;
 	uint no_mag            : 1;
+	uint keyboard          : 1;
 	enum output fmt;
 } Options;
 
@@ -383,7 +384,7 @@ version(void)
 {
 	char s[] =
 		PROGNAME" "VERSION"\n\n"
-		"Copyright (C) 2022-2023 NRK and contributors.\n"
+		"Copyright (C) 2022-2024 NRK and contributors.\n"
 		"License: GPLv3+ <https://gnu.org/licenses/gpl.html>.\n"
 		"Upstream: <https://codeberg.org/NRK/sxcs>\n"
 	;
@@ -445,6 +446,8 @@ opt_parse(int argc, char *argv[])
 			ret.oneshot = 1;
 		else if (str_eq(a, S("--quit-on-keypress")) || str_eq(a, S("-q")))
 			ret.quit_on_keypress = 1;
+		else if (str_eq(a, S("--keyboard")) || str_eq(a, S("-k")))
+			ret.keyboard = 1;
 		else if (str_eq(a, S("--mag-none")))
 			ret.no_mag = 1;
 		else if (str_eq(a, S("--mag-filters")))
@@ -460,6 +463,9 @@ opt_parse(int argc, char *argv[])
 	if (ret.fmt == OUTPUT_NONE && !no_color)
 		ret.fmt = OUTPUT_DEFAULT;
 
+	if (ret.quit_on_keypress && ret.keyboard)
+		fatal("--quit-on-keypress and --keyboard cannot be enabled at the same time");
+
 	return ret;
 }
 
@@ -601,8 +607,12 @@ magnify(const int x, const int y)
 	);
 	if (img.im == NULL)
 		fatal("failed to get image");
-	if (img.im->bits_per_pixel != 32) /* ximg_pixel_get() depends on it */
-		fatal("unexpected bits_per_pixel");
+	if (img.im->bits_per_pixel != 32 ||
+	    img.im->bytes_per_line != (img.im->width * 4) ||
+	    !(img.im->depth == 24 || img.im->depth == 32))
+	{ /* ximg_pixel_get() depends on these */
+		fatal("unexpected XImage format");
+	}
 	mag_func(cursor_img, &img);
 	XDestroyImage(img.im);
 
@@ -636,7 +646,7 @@ main(int argc, char *argv[])
 	if ((x11.dpy = XOpenDisplay(NULL)) == NULL)
 		fatal("failed to open x11 display");
 
-	{ /* TODO: update the x11.root.{w,h} if root changes size */
+	{
 		XWindowAttributes tmp;
 		x11.root.win = DefaultRootWindow(x11.dpy);
 		if (XGetWindowAttributes(x11.dpy, x11.root.win, &tmp) == 0)
@@ -668,7 +678,7 @@ main(int argc, char *argv[])
 		cursor_img->xhot = cursor_img->yhot = MAG_SIZE / 2;
 	}
 
-	if (opt.quit_on_keypress) {
+	if (opt.quit_on_keypress || opt.keyboard) {
 		/* when launched via dwm keybinding, it fails the grab since
 		 * dwm has it grabbed already. listen for FocusChangeMask and
 		 * keep retrying. */
@@ -769,10 +779,41 @@ main(int argc, char *argv[])
 			}
 			magnify(old.x, old.y);
 			break;
-		case KeyPress:
+		case KeyPress: {
+			KeySym k = None;
+			int x = ev.xkey.x_root, y = ev.xkey.y_root;
+			int delta = (ev.xkey.state & ControlMask) ? 1 :
+			            ((ev.xkey.state & ShiftMask) ? 128 : 16);
+
 			if (opt.quit_on_keypress)
 				goto out;
-			break;
+
+			if (opt.keyboard) {
+				char junk;
+				XLookupString(&ev.xkey, &junk, 1, &k, NULL);
+			}
+
+			switch (k) {
+			case XK_h: case XK_H: case XK_Left:  x -= delta; break;
+			case XK_l: case XK_L: case XK_Right: x += delta; break;
+			case XK_k: case XK_K: case XK_Up:    y -= delta; break;
+			case XK_j: case XK_J: case XK_Down:  y += delta; break;
+			case XK_q: case XK_Q: case XK_Escape: goto out; break;
+			case XK_minus: case XK_KP_Subtract:
+				MAG_FACTOR = MAX(1.1f, MAG_FACTOR / MAG_STEP);
+				break;
+			case XK_plus: case XK_KP_Add:
+				MAG_FACTOR *= MAG_STEP;
+				break;
+			case XK_space:
+				print_color(ev.xkey.x_root, ev.xkey.y_root, opt.fmt);
+				if (opt.oneshot)
+					goto out;
+				break;
+			}
+			if (x != ev.xkey.x_root || y != ev.xkey.y_root)
+				XWarpPointer(x11.dpy, None, x11.root.win, 0, 0, 0, 0, x, y);
+		} break;
 		default:
 			break;
 		}
